# check_dns.py -- Returns OK if a hostname resolves to any ip address
from _socket import gaierror
import socket
import time
from pynag.Plugins import PluginHelper,ok,warning,critical,unknown

my_plugin = PluginHelper()
my_plugin.parser.add_option('-H', help="Hostname or ip address", dest="hostname")
my_plugin.parser.add_option('-a', help="Expected Address", dest="address")
my_plugin.parse_arguments()


hostname = my_plugin.options.hostname
address = my_plugin.options.address
if hostname is None:
    my_plugin.parser.error('-H argument is required')

# Here comes the specific check logic
try:
    start_time = time.time()
    result = socket.gethostbyname( hostname )
    end_time = time.time()
    # If no address was specified with -a, then we return
    # OK if hostname resolved to anything at all
    if address is None or address == result:
        my_plugin.status(ok)
        my_plugin.add_summary("%s resolves to %s" % (hostname, result))
    else:
        my_plugin.status(critical)
        my_plugin.add_summary("%s resolves to %s but should resolve to %s" % (hostname,result,address))

    # Add run_time metric, so we can also alert if lookup takes to long
    run_time = end_time - start_time
    my_plugin.add_metric('run_time', run_time)
except gaierror:
    my_plugin.status(critical)
    my_plugin.add_summary('Could not resolve host "%s"' % hostname )

# Run check_all_metrics so end-user has the option of thresholding on run_time
my_plugin.check_all_metrics()

# Print status output and exit
my_plugin.exit()



# check_load.py - Check load average. Thresholds can be specified from the commandline

from pynag.Plugins import PluginHelper,ok,warning,critical,unknown


helper = PluginHelper()
helper.parse_arguments()

try:
    content = open('/proc/loadavg').read()
except Exception, e:
    helper.exit(summary="Could not read /proc/loadavg", long_output=str(e), exit_code=unknown, perfdata='')


helper.add_summary("Load: %s" % content)

# Read metrics from /proc/loadavg and add them as performance metrics
load1,load5,load15,processes,last_proc_id = content.split()
running,total = processes.split('/')

# If we so desire we can set default thresholds by adding warn attribute here
# However we decide that there are no thresholds by default and they have to be
# applied on runtime with the --threshold option
helper.add_metric(label='load1',value=load1)
helper.add_metric(label='load5',value=load5)
helper.add_metric(label='load15',value=load15)
helper.add_metric(label='running_processes',value=running)
helper.add_metric(label='total_processes',value=total)

# By default assume everything is ok:
helper.status(ok)

# Here all metrics will be checked against thresholds that are either
# built-in or added via --threshold from the command-line
helper.check_all_metrics()

# Print out plugin information and exit nagios-style
helper.exit()




def add_host_comment(host_name,persistent,author,comment,command_file=None,timestamp=0):
    """ Adds a comment to a particular host.  If the "persistent" field is set to zero (0), the comment will be deleted the next time Nagios is restarted.  Otherwise, the comment will persist across program restarts until it is deleted manually. """
    return send_command("ADD_HOST_COMMENT",command_file,timestamp, host_name,persistent,author,comment)

def shutdown_program(command_file=None,timestamp=0):
    """ Shuts down the Nagios process. """
    return send_command("SHUTDOWN_PROGRAM",command_file,timestamp, )

def disable_servicegroup_passive_svc_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Disables the acceptance and processing of passive checks for all services in a particular servicegroup. """
    return send_command("DISABLE_SERVICEGROUP_PASSIVE_SVC_CHECKS",command_file,timestamp, servicegroup_name)

def enable_servicegroup_passive_host_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Enables the acceptance and processing of passive checks for all hosts that have services that are members of a particular service group. """
    return send_command("ENABLE_SERVICEGROUP_PASSIVE_HOST_CHECKS",command_file,timestamp, servicegroup_name)

def disable_servicegroup_passive_host_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Disables the acceptance and processing of passive checks for all hosts that have services that are members of a particular service group. """
    return send_command("DISABLE_SERVICEGROUP_PASSIVE_HOST_CHECKS",command_file,timestamp, servicegroup_name)

def change_global_host_event_handler(event_handler_command,command_file=None,timestamp=0):
    """ Changes the global host event handler command to be that specified by the "event_handler_command" option.  The "event_handler_command" option specifies the short name of the command that should be used as the new host event handler.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_GLOBAL_HOST_EVENT_HANDLER",command_file,timestamp, event_handler_command)

def change_global_svc_event_handler(event_handler_command,command_file=None,timestamp=0):
    """ Changes the global service event handler command to be that specified by the "event_handler_command" option.  The "event_handler_command" option specifies the short name of the command that should be used as the new service event handler.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_GLOBAL_SVC_EVENT_HANDLER",command_file,timestamp, event_handler_command)

def change_host_event_handler(host_name,event_handler_command,command_file=None,timestamp=0):
    """ Changes the event handler command for a particular host to be that specified by the "event_handler_command" option.  The "event_handler_command" option specifies the short name of the command that should be used as the new host event handler.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_HOST_EVENT_HANDLER",command_file,timestamp, host_name,event_handler_command)

def change_svc_event_handler(host_name,service_description,event_handler_command,command_file=None,timestamp=0):
    """ Changes the event handler command for a particular service to be that specified by the "event_handler_command" option.  The "event_handler_command" option specifies the short name of the command that should be used as the new service event handler.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_SVC_EVENT_HANDLER",command_file,timestamp, host_name,service_description,event_handler_command)

def change_host_check_command(host_name,check_command,command_file=None,timestamp=0):
    """ Changes the check command for a particular host to be that specified by the "check_command" option.  The "check_command" option specifies the short name of the command that should be used as the new host check command.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_HOST_CHECK_COMMAND",command_file,timestamp, host_name,check_command)

def change_svc_check_command(host_name,service_description,check_command,command_file=None,timestamp=0):
    """ Changes the check command for a particular service to be that specified by the "check_command" option.  The "check_command" option specifies the short name of the command that should be used as the new service check command.  The command must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_SVC_CHECK_COMMAND",command_file,timestamp, host_name,service_description,check_command)

def change_normal_host_check_interval(host_name,check_interval,command_file=None,timestamp=0):
    """ Changes the normal (regularly scheduled) check interval for a particular host. """
    return send_command("CHANGE_NORMAL_HOST_CHECK_INTERVAL",command_file,timestamp, host_name,check_interval)

def enable_svc_notifications(host_name,service_description,command_file=None,timestamp=0):
    """ Enables notifications for a particular service.  Notifications will be sent out for the service only if notifications are enabled on a program-wide basis as well. """
    return send_command("ENABLE_SVC_NOTIFICATIONS",command_file,timestamp, host_name,service_description)

def change_normal_svc_check_interval(host_name,service_description,check_interval,command_file=None,timestamp=0):
    """ Changes the normal (regularly scheduled) check interval for a particular service """
    return send_command("CHANGE_NORMAL_SVC_CHECK_INTERVAL",command_file,timestamp, host_name,service_description,check_interval)

def change_retry_svc_check_interval(host_name,service_description,check_interval,command_file=None,timestamp=0):
    """ Changes the retry check interval for a particular service. """
    return send_command("CHANGE_RETRY_SVC_CHECK_INTERVAL",command_file,timestamp, host_name,service_description,check_interval)

def change_max_host_check_attempts(host_name,check_attempts,command_file=None,timestamp=0):
    """ Changes the maximum number of check attempts (retries) for a particular host. """
    return send_command("CHANGE_MAX_HOST_CHECK_ATTEMPTS",command_file,timestamp, host_name,check_attempts)

def change_max_svc_check_attempts(host_name,service_description,check_attempts,command_file=None,timestamp=0):
    """ Changes the maximum number of check attempts (retries) for a particular service. """
    return send_command("CHANGE_MAX_SVC_CHECK_ATTEMPTS",command_file,timestamp, host_name,service_description,check_attempts)

def process_service_check_result(host_name,service_description,return_code,plugin_output,command_file=None,timestamp=0):
    """ This is used to submit a passive check result for a particular service.  The "return_code" field should be one of the following: 0=OK, 1=WARNING, 2=CRITICAL, 3=UNKNOWN.  The "plugin_output" field contains text output from the service check, along with optional performance data. """
    return send_command("PROCESS_SERVICE_CHECK_RESULT",command_file,timestamp, host_name,service_description,return_code,plugin_output)

def process_host_check_result(host_name,status_code,plugin_output,command_file=None,timestamp=0):
    """ This is used to submit a passive check result for a particular host.  The "status_code" indicates the state of the host check and should be one of the following: 0=UP, 1=DOWN, 2=UNREACHABLE.  The "plugin_output" argument contains the text returned from the host check, along with optional performance data. """
    return send_command("PROCESS_HOST_CHECK_RESULT",command_file,timestamp, host_name,status_code,plugin_output)

def remove_host_acknowledgement(host_name,command_file=None,timestamp=0):
    """ This removes the problem acknowledgement for a particular host.  Once the acknowledgement has been removed, notifications can once again be sent out for the given host. """
    return send_command("REMOVE_HOST_ACKNOWLEDGEMENT",command_file,timestamp, host_name)

def remove_svc_acknowledgement(host_name,service_description,command_file=None,timestamp=0):
    """ This removes the problem acknowledgement for a particular service.  Once the acknowledgement has been removed, notifications can once again be sent out for the given service. """
    return send_command("REMOVE_SVC_ACKNOWLEDGEMENT",command_file,timestamp, host_name,service_description)

def schedule_host_downtime(host_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for a specified host.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The specified host downtime can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the specified host should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_HOST_DOWNTIME",command_file,timestamp, host_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_svc_downtime(host_name,service_desriptionstart_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for a specified service.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The specified service downtime can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the specified service should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_SVC_DOWNTIME",command_file,timestamp, host_name,service_desriptionstart_time,end_time,fixed,trigger_id,duration,author,comment)

def disable_svc_notifications(host_name,service_description,command_file=None,timestamp=0):
    """ Disables notifications for a particular service. """
    return send_command("DISABLE_SVC_NOTIFICATIONS",command_file,timestamp, host_name,service_description)

def schedule_servicegroup_svc_downtime(servicegroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for all services in a specified servicegroup.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The service downtime entries can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the services should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_SERVICEGROUP_SVC_DOWNTIME",command_file,timestamp, servicegroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_servicegroup_host_downtime(servicegroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for all hosts that have services in a specified servicegroup.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The host downtime entries can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the hosts should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_SERVICEGROUP_HOST_DOWNTIME",command_file,timestamp, servicegroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_host_svc_downtime(host_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for all services associated with a particular host.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The service downtime entries can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the services should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_HOST_SVC_DOWNTIME",command_file,timestamp, host_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_hostgroup_host_downtime(hostgroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for all hosts in a specified hostgroup.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The host downtime entries can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the hosts should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_HOSTGROUP_HOST_DOWNTIME",command_file,timestamp, hostgroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_hostgroup_svc_downtime(hostgroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for all services associated with hosts in a specified servicegroup.  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The service downtime entries can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the services should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_HOSTGROUP_SVC_DOWNTIME",command_file,timestamp, hostgroup_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def del_host_downtime(downtime_id,command_file=None,timestamp=0):
    """ Deletes the host downtime entry that has an ID number matching the "downtime_id" argument.  If the downtime is currently in effect, the host will come out of scheduled downtime (as long as there are no other overlapping active downtime entries). """
    return send_command("DEL_HOST_DOWNTIME",command_file,timestamp, downtime_id)

def del_svc_downtime(downtime_id,command_file=None,timestamp=0):
    """ Deletes the service downtime entry that has an ID number matching the "downtime_id" argument.  If the downtime is currently in effect, the service will come out of scheduled downtime (as long as there are no other overlapping active downtime entries). """
    return send_command("DEL_SVC_DOWNTIME",command_file,timestamp, downtime_id)

def schedule_host_check(host_name,check_time,command_file=None,timestamp=0):
    """ Schedules the next active check of a particular host at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).  Note that the host may not actually be checked at the time you specify.  This could occur for a number of reasons: active checks are disabled on a program-wide or service-specific basis, the host is already scheduled to be checked at an earlier time, etc.  If you want to force the host check to occur at the time you specify, look at the SCHEDULE_FORCED_HOST_CHECK command. """
    return send_command("SCHEDULE_HOST_CHECK",command_file,timestamp, host_name,check_time)

def schedule_forced_host_check(host_name,check_time,command_file=None,timestamp=0):
    """ Schedules a forced active check of a particular host at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).   Forced checks are performed regardless of what time it is (e.g. timeperiod restrictions are ignored) and whether or not active checks are enabled on a host-specific or program-wide basis. """
    return send_command("SCHEDULE_FORCED_HOST_CHECK",command_file,timestamp, host_name,check_time)

def schedule_forced_svc_check(host_name,service_description,check_time,command_file=None,timestamp=0):
    """ Schedules a forced active check of a particular service at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).   Forced checks are performed regardless of what time it is (e.g. timeperiod restrictions are ignored) and whether or not active checks are enabled on a service-specific or program-wide basis. """
    return send_command("SCHEDULE_FORCED_SVC_CHECK",command_file,timestamp, host_name,service_description,check_time)

def del_all_host_comments(host_name,command_file=None,timestamp=0):
    """ Deletes all comments assocated with a particular host. """
    return send_command("DEL_ALL_HOST_COMMENTS",command_file,timestamp, host_name)

def schedule_forced_host_svc_checks(host_name,check_time,command_file=None,timestamp=0):
    """ Schedules a forced active check of all services associated with a particular host at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).   Forced checks are performed regardless of what time it is (e.g. timeperiod restrictions are ignored) and whether or not active checks are enabled on a service-specific or program-wide basis. """
    return send_command("SCHEDULE_FORCED_HOST_SVC_CHECKS",command_file,timestamp, host_name,check_time)

def process_file(file_name,delete,command_file=None,timestamp=0):
    """ Directs Nagios to process all external commands that are found in the file specified by the <file_name> argument.  If the <delete> option is non-zero, the file will be deleted once it has been processes.  If the <delete> option is set to zero, the file is left untouched. """
    return send_command("PROCESS_FILE",command_file,timestamp, file_name,delete)

def change_host_check_timeperiod(host_name,check_timeperod,command_file=None,timestamp=0):
    """ Changes the check timeperiod for a particular host to what is specified by the "check_timeperiod" option.  The "check_timeperiod" option should be the short name of the timeperod that is to be used as the host check timeperiod.  The timeperiod must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_HOST_CHECK_TIMEPERIOD",command_file,timestamp, host_name,check_timeperod)

def send_custom_host_notification(host_name,options,author,comment,command_file=None,timestamp=0):
    """ Allows you to send a custom host notification.  Very useful in dire situations, emergencies or to communicate with all admins that are responsible for a particular host.  When the host notification is sent out, the $NOTIFICATIONTYPE$ macro will be set to "CUSTOM".  The <options> field is a logical OR of the following integer values that affect aspects of the notification that are sent out: 0 = No option (default), 1 = Broadcast (send notification to all normal and all escalated contacts for the host), 2 = Forced (notification is sent out regardless of current time, whether or not notifications are enabled, etc.), 4 = Increment current notification # for the host (this is not done by default for custom notifications).  The comment field can be used with the
$NOTIFICATIONCOMMENT$ macro in notification commands. """
    return send_command("SEND_CUSTOM_HOST_NOTIFICATION",command_file,timestamp, host_name,options,author,comment)

def send_custom_svc_notification(host_name,service_description,options,author,comment,command_file=None,timestamp=0):
    """ Allows you to send a custom service notification.  Very useful in dire situations, emergencies or to communicate with all admins that are responsible for a particular service.  When the service notification is sent out, the $NOTIFICATIONTYPE$ macro will be set to "CUSTOM".  The <options> field is a logical OR of the following integer values that affect aspects of the notification that are sent out: 0 = No option (default), 1 = Broadcast (send notification to all normal and all escalated contacts for the service), 2 = Forced (notification is sent out regardless of current time, whether or not notifications are enabled, etc.), 4 = Increment current notification # for the service(this is not done by default for custom notifications).   The comment field can be used with the
$NOTIFICATIONCOMMENT$ macro in notification commands. """
    return send_command("SEND_CUSTOM_SVC_NOTIFICATION",command_file,timestamp, host_name,service_description,options,author,comment)

def change_retry_host_check_interval(host_name,service_description,check_interval,command_file=None,timestamp=0):
    """ Changes the retry check interval for a particular host. """
    return send_command("CHANGE_RETRY_HOST_CHECK_INTERVAL",command_file,timestamp, host_name,service_description,check_interval)

def change_svc_check_timeperiod(host_name,service_description,check_timeperiod,command_file=None,timestamp=0):
    """ Changes the check timeperiod for a particular service to what is specified by the "check_timeperiod" option.  The "check_timeperiod" option should be the short name of the timeperod that is to be used as the service check timeperiod.  The timeperiod must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_SVC_CHECK_TIMEPERIOD",command_file,timestamp, host_name,service_description,check_timeperiod)

def change_host_check_timeperiod(host_name,timeperiod,command_file=None,timestamp=0):
    """ Changes the valid check period for the specified host. """
    return send_command("CHANGE_HOST_CHECK_TIMEPERIOD",command_file,timestamp, host_name,timeperiod)

def change_custom_host_var(host_name,varname,varvalue,command_file=None,timestamp=0):
    """ Changes the value of a custom host variable. """
    return send_command("CHANGE_CUSTOM_HOST_VAR",command_file,timestamp, host_name,varname,varvalue)

def del_all_svc_comments(host_name,service_description,command_file=None,timestamp=0):
    """ Deletes all comments associated with a particular service. """
    return send_command("DEL_ALL_SVC_COMMENTS",command_file,timestamp, host_name,service_description)

def change_custom_svc_var(host_name,service_description,varname,varvalue,command_file=None,timestamp=0):
    """ Changes the value of a custom service variable. """
    return send_command("CHANGE_CUSTOM_SVC_VAR",command_file,timestamp, host_name,service_description,varname,varvalue)

def change_custom_contact_var(contact_name,varname,varvalue,command_file=None,timestamp=0):
    """ Changes the value of a custom contact variable. """
    return send_command("CHANGE_CUSTOM_CONTACT_VAR",command_file,timestamp, contact_name,varname,varvalue)

def enable_contact_host_notifications(contact_name,command_file=None,timestamp=0):
    """ Enables host notifications for a particular contact. """
    return send_command("ENABLE_CONTACT_HOST_NOTIFICATIONS",command_file,timestamp, contact_name)

def disable_contact_host_notifications(contact_name,command_file=None,timestamp=0):
    """ Disables host notifications for a particular contact. """
    return send_command("DISABLE_CONTACT_HOST_NOTIFICATIONS",command_file,timestamp, contact_name)

def enable_contact_svc_notifications(contact_name,command_file=None,timestamp=0):
    """ Disables service notifications for a particular contact. """
    return send_command("ENABLE_CONTACT_SVC_NOTIFICATIONS",command_file,timestamp, contact_name)

def disable_contact_svc_notifications(contact_name,command_file=None,timestamp=0):
    """ Disables service notifications for a particular contact. """
    return send_command("DISABLE_CONTACT_SVC_NOTIFICATIONS",command_file,timestamp, contact_name)

def enable_contactgroup_host_notifications(contactgroup_name,command_file=None,timestamp=0):
    """ Enables host notifications for all contacts in a particular contactgroup. """
    return send_command("ENABLE_CONTACTGROUP_HOST_NOTIFICATIONS",command_file,timestamp, contactgroup_name)

def disable_contactgroup_host_notifications(contactgroup_name,command_file=None,timestamp=0):
    """ Disables host notifications for all contacts in a particular contactgroup. """
    return send_command("DISABLE_CONTACTGROUP_HOST_NOTIFICATIONS",command_file,timestamp, contactgroup_name)

def enable_contactgroup_svc_notifications(contactgroup_name,command_file=None,timestamp=0):
    """ Enables service notifications for all contacts in a particular contactgroup. """
    return send_command("ENABLE_CONTACTGROUP_SVC_NOTIFICATIONS",command_file,timestamp, contactgroup_name)

def disable_contactgroup_svc_notifications(contactgroup_name,command_file=None,timestamp=0):
    """ Disables service notifications for all contacts in a particular contactgroup. """
    return send_command("DISABLE_CONTACTGROUP_SVC_NOTIFICATIONS",command_file,timestamp, contactgroup_name)

def enable_host_notifications(host_name,command_file=None,timestamp=0):
    """ Enables notifications for a particular host.  Notifications will be sent out for the host only if notifications are enabled on a program-wide basis as well. """
    return send_command("ENABLE_HOST_NOTIFICATIONS",command_file,timestamp, host_name)

def disable_svc_flap_detection(host_name,service_description,command_file=None,timestamp=0):
    """ Disables flap detection for the specified service. """
    return send_command("DISABLE_SVC_FLAP_DETECTION",command_file,timestamp, host_name,service_description)

def change_svc_notification_timeperiod(host_name,service_description,notification_timeperiod,command_file=None,timestamp=0):
    """ Changes the notification timeperiod for a particular service to what is specified by the "notification_timeperiod" option.  The "notification_timeperiod" option should be the short name of the timeperiod that is to be used as the service notification timeperiod.  The timeperiod must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_SVC_NOTIFICATION_TIMEPERIOD",command_file,timestamp, host_name,service_description,notification_timeperiod)

def change_contact_svc_notification_timeperiod(contact_name,notification_timeperiod,command_file=None,timestamp=0):
    """ Changes the service notification timeperiod for a particular contact to what is specified by the "notification_timeperiod" option.  The "notification_timeperiod" option should be the short name of the timeperiod that is to be used as the contact's service notification timeperiod.  The timeperiod must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_CONTACT_SVC_NOTIFICATION_TIMEPERIOD",command_file,timestamp, contact_name,notification_timeperiod)

def change_contact_host_notification_timeperiod(contact_name,notification_timeperiod,command_file=None,timestamp=0):
    """ Changes the host notification timeperiod for a particular contact to what is specified by the "notification_timeperiod" option.  The "notification_timeperiod" option should be the short name of the timeperiod that is to be used as the contact's host notification timeperiod.  The timeperiod must have been configured in Nagios before it was last (re)started. """
    return send_command("CHANGE_CONTACT_HOST_NOTIFICATION_TIMEPERIOD",command_file,timestamp, contact_name,notification_timeperiod)

def change_host_modattr(host_name,value,command_file=None,timestamp=0):
    """ This command changes the modified attributes value for the specified host.  Modified attributes values are used by Nagios to determine which object properties should be retained across program restarts.  Thus, modifying the value of the attributes can affect data retention.  This is an advanced option and should only be used by people who are intimately familiar with the data retention logic in Nagios. """
    return send_command("CHANGE_HOST_MODATTR",command_file,timestamp, host_name,value)

def change_svc_modattr(host_name,service_description,value,command_file=None,timestamp=0):
    """ This command changes the modified attributes value for the specified service.  Modified attributes values are used by Nagios to determine which object properties should be retained across program restarts.  Thus, modifying the value of the attributes can affect data retention.  This is an advanced option and should only be used by people who are intimately familiar with the data retention logic in Nagios. """
    return send_command("CHANGE_SVC_MODATTR",command_file,timestamp, host_name,service_description,value)

def change_contact_modattr(contact_name,value,command_file=None,timestamp=0):
    """ This command changes the modified attributes value for the specified contact.  Modified attributes values are used by Nagios to determine which object properties should be retained across program restarts.  Thus, modifying the value of the attributes can affect data retention.  This is an advanced option and should only be used by people who are intimately familiar with the data retention logic in Nagios. """
    return send_command("CHANGE_CONTACT_MODATTR",command_file,timestamp, contact_name,value)

def change_contact_modhattr(contact_name,value,command_file=None,timestamp=0):
    """ This command changes the modified host attributes value for the specified contact.  Modified attributes values are used by Nagios to determine which object properties should be retained across program restarts.  Thus, modifying the value of the attributes can affect data retention.  This is an advanced option and should only be used by people who are intimately familiar with the data retention logic in Nagios. """
    return send_command("CHANGE_CONTACT_MODHATTR",command_file,timestamp, contact_name,value)

def change_contact_modsattr(contact_name,value,command_file=None,timestamp=0):
    """ This command changes the modified service attributes value for the specified contact.  Modified attributes values are used by Nagios to determine which object properties should be retained across program restarts.  Thus, modifying the value of the attributes can affect data retention.  This is an advanced option and should only be used by people who are intimately familiar with the data retention logic in Nagios. """
    return send_command("CHANGE_CONTACT_MODSATTR",command_file,timestamp, contact_name,value)

def disable_host_notifications(host_name,command_file=None,timestamp=0):
    """ Disables notifications for a particular host. """
    return send_command("DISABLE_HOST_NOTIFICATIONS",command_file,timestamp, host_name)

def enable_all_notifications_beyond_host(host_name,command_file=None,timestamp=0):
    """ Enables notifications for all hosts and services "beyond" (e.g. on all child hosts of) the specified host.  The current notification setting for the specified host is not affected.  Notifications will only be sent out for these hosts and services if notifications are also enabled on a program-wide basis. """
    return send_command("ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST",command_file,timestamp, host_name)

def disable_all_notifications_beyond_host(host_name,command_file=None,timestamp=0):
    """ Disables notifications for all hosts and services "beyond" (e.g. on all child hosts of) the specified host.  The current notification setting for the specified host is not affected. """
    return send_command("DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST",command_file,timestamp, host_name)

def enable_host_and_child_notifications(host_name,command_file=None,timestamp=0):
    """ Enables notifications for the specified host, as well as all hosts "beyond" (e.g. on all child hosts of) the specified host.  Notifications will only be sent out for these hosts if notifications are also enabled on a program-wide basis. """
    return send_command("ENABLE_HOST_AND_CHILD_NOTIFICATIONS",command_file,timestamp, host_name)

def add_svc_comment(host_name,service_description,persistent,author,comment,command_file=None,timestamp=0):
    """ Adds a comment to a particular service.  If the "persistent" field is set to zero (0), the comment will be deleted the next time Nagios is restarted.  Otherwise, the comment will persist across program restarts until it is deleted manually. """
    return send_command("ADD_SVC_COMMENT",command_file,timestamp, host_name,service_description,persistent,author,comment)

def disable_host_and_child_notifications(host_name,command_file=None,timestamp=0):
    """ Disables notifications for the specified host, as well as all hosts "beyond" (e.g. on all child hosts of) the specified host. """
    return send_command("DISABLE_HOST_AND_CHILD_NOTIFICATIONS",command_file,timestamp, host_name)

def set_host_notification_number(host_name,notification_number,command_file=None,timestamp=0):
    """ Sets the current notification number for a particular host.  A value of 0 indicates that no notification has yet been sent for the current host problem.  Useful for forcing an escalation (based on notification number) or replicating notification information in redundant monitoring environments. Notification numbers greater than zero have no noticeable affect on the notification process if the host is currently in an UP state. """
    return send_command("SET_HOST_NOTIFICATION_NUMBER",command_file,timestamp, host_name,notification_number)

def set_svc_notification_number(host_name,service_description,notification_number,command_file=None,timestamp=0):
    """ Sets the current notification number for a particular service.  A value of 0 indicates that no notification has yet been sent for the current service problem.  Useful for forcing an escalation (based on notification number) or replicating notification information in redundant monitoring environments. Notification numbers greater than zero have no noticeable affect on the notification process if the service is currently in an OK state. """
    return send_command("SET_SVC_NOTIFICATION_NUMBER",command_file,timestamp, host_name,service_description,notification_number)

def enable_service_freshness_checks(command_file=None,timestamp=0):
    """ Enables freshness checks of all services on a program-wide basis.  Individual services that have freshness checks disabled will not be checked for freshness. """
    return send_command("ENABLE_SERVICE_FRESHNESS_CHECKS",command_file,timestamp, )

def enable_host_freshness_checks(command_file=None,timestamp=0):
    """ Enables freshness checks of all hosts on a program-wide basis.  Individual hosts that have freshness checks disabled will not be checked for freshness. """
    return send_command("ENABLE_HOST_FRESHNESS_CHECKS",command_file,timestamp, )

def disable_service_freshness_checks(command_file=None,timestamp=0):
    """ Disables freshness checks of all services on a program-wide basis. """
    return send_command("DISABLE_SERVICE_FRESHNESS_CHECKS",command_file,timestamp, )

def disable_host_freshness_checks(command_file=None,timestamp=0):
    """ Disables freshness checks of all hosts on a program-wide basis. """
    return send_command("DISABLE_HOST_FRESHNESS_CHECKS",command_file,timestamp, )

def schedule_and_propagate_triggered_host_downtime(host_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for a specified host and all of its children (hosts).  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  Downtime for child hosts are all set to be triggered by the downtime for the specified (parent) host.  The specified (parent) host downtime can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the specified (parent) host should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_AND_PROPAGATE_TRIGGERED_HOST_DOWNTIME",command_file,timestamp, host_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_and_propagate_host_downtime(host_name,start_time,end_time,fixed,trigger_id,duration,author,comment,command_file=None,timestamp=0):
    """ Schedules downtime for a specified host and all of its children (hosts).  If the "fixed" argument is set to one (1), downtime will start and end at the times specified by the "start" and "end" arguments.  Otherwise, downtime will begin between the "start" and "end" times and last for "duration" seconds.  The "start" and "end" arguments are specified in time_t format (seconds since the UNIX epoch).  The specified (parent) host downtime can be triggered by another downtime entry if the "trigger_id" is set to the ID of another scheduled downtime entry.  Set the "trigger_id" argument to zero (0) if the downtime for the specified (parent) host should not be triggered by another downtime entry. """
    return send_command("SCHEDULE_AND_PROPAGATE_HOST_DOWNTIME",command_file,timestamp, host_name,start_time,end_time,fixed,trigger_id,duration,author,comment)

def schedule_svc_check(host_name,service_description,check_time,command_file=None,timestamp=0):
    """ Schedules the next active check of a specified service at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).  Note that the service may not actually be checked at the time you specify.  This could occur for a number of reasons: active checks are disabled on a program-wide or service-specific basis, the service is already scheduled to be checked at an earlier time, etc.  If you want to force the service check to occur at the time you specify, look at the SCHEDULE_FORCED_SVC_CHECK command. """
    return send_command("SCHEDULE_SVC_CHECK",command_file,timestamp, host_name,service_description,check_time)

def del_host_comment(comment_id,command_file=None,timestamp=0):
    """ Deletes a host comment.  The id number of the comment that is to be deleted must be specified. """
    return send_command("DEL_HOST_COMMENT",command_file,timestamp, comment_id)

def schedule_host_svc_checks(host_name,check_time,command_file=None,timestamp=0):
    """ Schedules the next active check of all services on a particular host at "check_time".  The "check_time" argument is specified in time_t format (seconds since the UNIX epoch).  Note that the services may not actually be checked at the time you specify.  This could occur for a number of reasons: active checks are disabled on a program-wide or service-specific basis, the services are already scheduled to be checked at an earlier time, etc.  If you want to force the service checks to occur at the time you specify, look at the SCHEDULE_FORCED_HOST_SVC_CHECKS command. """
    return send_command("SCHEDULE_HOST_SVC_CHECKS",command_file,timestamp, host_name,check_time)

def save_state_information(command_file=None,timestamp=0):
    """ Causes Nagios to save all current monitoring status information to the state retention file.  Normally, state retention information is saved before the Nagios process shuts down and (potentially) at regularly scheduled intervals.  This command allows you to force Nagios to save this information to the state retention file immediately.  This does not affect the current status information in the Nagios process. """
    return send_command("SAVE_STATE_INFORMATION",command_file,timestamp, )

def read_state_information(command_file=None,timestamp=0):
    """ Causes Nagios to load all current monitoring status information from the state retention file.  Normally, state retention information is loaded when the Nagios process starts up and before it starts monitoring.  WARNING: This command will cause Nagios to discard all current monitoring status information and use the information stored in state retention file!  Use with care. """
    return send_command("READ_STATE_INFORMATION",command_file,timestamp, )

def enable_host_svc_checks(host_name,command_file=None,timestamp=0):
    """ Enables active checks of all services on the specified host. """
    return send_command("ENABLE_HOST_SVC_CHECKS",command_file,timestamp, host_name)

def disable_host_svc_checks(host_name,command_file=None,timestamp=0):
    """ Enables active checks of all services on the specified host. """
    return send_command("DISABLE_HOST_SVC_CHECKS",command_file,timestamp, host_name)

def enable_host_svc_notifications(host_name,command_file=None,timestamp=0):
    """ Enables notifications for all services on the specified host.  Note that notifications will not be sent out if notifications are disabled on a program-wide basis. """
    return send_command("ENABLE_HOST_SVC_NOTIFICATIONS",command_file,timestamp, host_name)

def disable_host_svc_notifications(host_name,command_file=None,timestamp=0):
    """ Disables notifications for all services on the specified host. """
    return send_command("DISABLE_HOST_SVC_NOTIFICATIONS",command_file,timestamp, host_name)

def delay_svc_notification(host_name,service_description,notification_time,command_file=None,timestamp=0):
    """ Delays the next notification for a parciular service until "notification_time".  The "notification_time" argument is specified in time_t format (seconds since the UNIX epoch).  Note that this will only have an affect if the service stays in the same problem state that it is currently in.  If the service changes to another state, a new notification may go out before the time you specify in the "notification_time" argument. """
    return send_command("DELAY_SVC_NOTIFICATION",command_file,timestamp, host_name,service_description,notification_time)

def delay_host_notification(host_name,notification_time,command_file=None,timestamp=0):
    """ Delays the next notification for a parciular service until "notification_time".  The "notification_time" argument is specified in time_t format (seconds since the UNIX epoch).  Note that this will only have an affect if the service stays in the same problem state that it is currently in.  If the service changes to another state, a new notification may go out before the time you specify in the "notification_time" argument. """
    return send_command("DELAY_HOST_NOTIFICATION",command_file,timestamp, host_name,notification_time)

def acknowledge_host_problem(host_name,sticky,notify,persistent,author,comment,command_file=None,timestamp=0):
    """ Allows you to acknowledge the current problem for the specified host.  By acknowledging the current problem, future notifications (for the same host state) are disabled.  If the "sticky" option is set to one (1), the acknowledgement will remain until the host returns to an UP state.  Otherwise the acknowledgement will automatically be removed when the host changes state.  If the "notify" option is set to one (1), a notification will be sent out to contacts indicating that the current host problem has been acknowledged.  If the "persistent" option is set to one (1), the comment associated with the acknowledgement will survive across restarts of the Nagios process.  If not, the comment will be deleted the next time Nagios restarts. """
    return send_command("ACKNOWLEDGE_HOST_PROBLEM",command_file,timestamp, host_name,sticky,notify,persistent,author,comment)

def del_svc_comment(comment_id,command_file=None,timestamp=0):
    """ Deletes a service comment.  The id number of the comment that is to be deleted must be specified. """
    return send_command("DEL_SVC_COMMENT",command_file,timestamp, comment_id)

def acknowledge_svc_problem(host_name,service_description,sticky,notify,persistent,author,comment,command_file=None,timestamp=0):
    """ Allows you to acknowledge the current problem for the specified service.  By acknowledging the current problem, future notifications (for the same servicestate) are disabled.  If the "sticky" option is set to one (1), the acknowledgement will remain until the service returns to an OK state.  Otherwise the acknowledgement will automatically be removed when the service changes state.  If the "notify" option is set to one (1), a notification will be sent out to contacts indicating that the current service problem has been acknowledged.  If the "persistent" option is set to one (1), the comment associated with the acknowledgement will survive across restarts of the Nagios process.  If not, the comment will be deleted the next time Nagios restarts. """
    return send_command("ACKNOWLEDGE_SVC_PROBLEM",command_file,timestamp, host_name,service_description,sticky,notify,persistent,author,comment)

def start_executing_svc_checks(command_file=None,timestamp=0):
    """ Enables active checks of services on a program-wide basis. """
    return send_command("START_EXECUTING_SVC_CHECKS",command_file,timestamp, )

def stop_executing_svc_checks(command_file=None,timestamp=0):
    """ Disables active checks of services on a program-wide basis. """
    return send_command("STOP_EXECUTING_SVC_CHECKS",command_file,timestamp, )

def start_accepting_passive_svc_checks(command_file=None,timestamp=0):
    """ Enables passive service checks on a program-wide basis. """
    return send_command("START_ACCEPTING_PASSIVE_SVC_CHECKS",command_file,timestamp, )

def stop_accepting_passive_svc_checks(command_file=None,timestamp=0):
    """ Disables passive service checks on a program-wide basis. """
    return send_command("STOP_ACCEPTING_PASSIVE_SVC_CHECKS",command_file,timestamp, )

def enable_passive_svc_checks(host_name,service_description,command_file=None,timestamp=0):
    """ Enables passive checks for the specified service. """
    return send_command("ENABLE_PASSIVE_SVC_CHECKS",command_file,timestamp, host_name,service_description)

def disable_passive_svc_checks(host_name,service_description,command_file=None,timestamp=0):
    """ Disables passive checks for the specified service. """
    return send_command("DISABLE_PASSIVE_SVC_CHECKS",command_file,timestamp, host_name,service_description)

def enable_event_handlers(command_file=None,timestamp=0):
    """ Enables host and service event handlers on a program-wide basis. """
    return send_command("ENABLE_EVENT_HANDLERS",command_file,timestamp, )

def disable_event_handlers(command_file=None,timestamp=0):
    """ Disables host and service event handlers on a program-wide basis. """
    return send_command("DISABLE_EVENT_HANDLERS",command_file,timestamp, )

def enable_host_event_handler(host_name,command_file=None,timestamp=0):
    """ Enables the event handler for the specified host. """
    return send_command("ENABLE_HOST_EVENT_HANDLER",command_file,timestamp, host_name)

def enable_svc_check(host_name,service_description,command_file=None,timestamp=0):
    """ Enables active checks for a particular service. """
    return send_command("ENABLE_SVC_CHECK",command_file,timestamp, host_name,service_description)

def disable_host_event_handler(host_name,command_file=None,timestamp=0):
    """ Disables the event handler for the specified host. """
    return send_command("DISABLE_HOST_EVENT_HANDLER",command_file,timestamp, host_name)

def enable_svc_event_handler(host_name,service_description,command_file=None,timestamp=0):
    """ Enables the event handler for the specified service. """
    return send_command("ENABLE_SVC_EVENT_HANDLER",command_file,timestamp, host_name,service_description)

def disable_svc_event_handler(host_name,service_description,command_file=None,timestamp=0):
    """ Disables the event handler for the specified service. """
    return send_command("DISABLE_SVC_EVENT_HANDLER",command_file,timestamp, host_name,service_description)

def enable_host_check(host_name,command_file=None,timestamp=0):
    """ Enables (regularly scheduled and on-demand) active checks of the specified host. """
    return send_command("ENABLE_HOST_CHECK",command_file,timestamp, host_name)

def disable_host_check(host_name,command_file=None,timestamp=0):
    """ Disables (regularly scheduled and on-demand) active checks of the specified host. """
    return send_command("DISABLE_HOST_CHECK",command_file,timestamp, host_name)

def start_obsessing_over_svc_checks(command_file=None,timestamp=0):
    """ Enables processing of service checks via the OCSP command on a program-wide basis. """
    return send_command("START_OBSESSING_OVER_SVC_CHECKS",command_file,timestamp, )

def stop_obsessing_over_svc_checks(command_file=None,timestamp=0):
    """ Disables processing of service checks via the OCSP command on a program-wide basis. """
    return send_command("STOP_OBSESSING_OVER_SVC_CHECKS",command_file,timestamp, )

def start_obsessing_over_host_checks(command_file=None,timestamp=0):
    """ Enables processing of host checks via the OCHP command on a program-wide basis. """
    return send_command("START_OBSESSING_OVER_HOST_CHECKS",command_file,timestamp, )

def stop_obsessing_over_host_checks(command_file=None,timestamp=0):
    """ Disables processing of host checks via the OCHP command on a program-wide basis. """
    return send_command("STOP_OBSESSING_OVER_HOST_CHECKS",command_file,timestamp, )

def start_obsessing_over_host(host_name,command_file=None,timestamp=0):
    """ Enables processing of host checks via the OCHP command for the specified host. """
    return send_command("START_OBSESSING_OVER_HOST",command_file,timestamp, host_name)

def disable_svc_check(host_name,service_description,command_file=None,timestamp=0):
    """ Disables active checks for a particular service. """
    return send_command("DISABLE_SVC_CHECK",command_file,timestamp, host_name,service_description)

def stop_obsessing_over_host(host_name,command_file=None,timestamp=0):
    """ Disables processing of host checks via the OCHP command for the specified host. """
    return send_command("STOP_OBSESSING_OVER_HOST",command_file,timestamp, host_name)

def start_obsessing_over_svc(host_name,service_description,command_file=None,timestamp=0):
    """ Enables processing of service checks via the OCSP command for the specified service. """
    return send_command("START_OBSESSING_OVER_SVC",command_file,timestamp, host_name,service_description)

def stop_obsessing_over_svc(host_name,service_description,command_file=None,timestamp=0):
    """ Disables processing of service checks via the OCSP command for the specified service. """
    return send_command("STOP_OBSESSING_OVER_SVC",command_file,timestamp, host_name,service_description)

def enable_failure_prediction(command_file=None,timestamp=0):
    """ Enables failure prediction on a program-wide basis.  This feature is not currently implemented in Nagios. """
    return send_command("ENABLE_FAILURE_PREDICTION",command_file,timestamp, )

def disable_failure_prediction(command_file=None,timestamp=0):
    """ Disables failure prediction on a program-wide basis.  This feature is not currently implemented in Nagios. """
    return send_command("DISABLE_FAILURE_PREDICTION",command_file,timestamp, )

def enable_performance_data(command_file=None,timestamp=0):
    """ Enables the processing of host and service performance data on a program-wide basis. """
    return send_command("ENABLE_PERFORMANCE_DATA",command_file,timestamp, )

def disable_performance_data(command_file=None,timestamp=0):
    """ Disables the processing of host and service performance data on a program-wide basis. """
    return send_command("DISABLE_PERFORMANCE_DATA",command_file,timestamp, )

def start_executing_host_checks(command_file=None,timestamp=0):
    """ Enables active host checks on a program-wide basis. """
    return send_command("START_EXECUTING_HOST_CHECKS",command_file,timestamp, )

def stop_executing_host_checks(command_file=None,timestamp=0):
    """ Disables active host checks on a program-wide basis. """
    return send_command("STOP_EXECUTING_HOST_CHECKS",command_file,timestamp, )

def start_accepting_passive_host_checks(command_file=None,timestamp=0):
    """ Enables acceptance and processing of passive host checks on a program-wide basis. """
    return send_command("START_ACCEPTING_PASSIVE_HOST_CHECKS",command_file,timestamp, )

def disable_notifications(command_file=None,timestamp=0):
    """ Disables host and service notifications on a program-wide basis. """
    return send_command("DISABLE_NOTIFICATIONS",command_file,timestamp, )

def stop_accepting_passive_host_checks(command_file=None,timestamp=0):
    """ Disables acceptance and processing of passive host checks on a program-wide basis. """
    return send_command("STOP_ACCEPTING_PASSIVE_HOST_CHECKS",command_file,timestamp, )

def enable_passive_host_checks(host_name,command_file=None,timestamp=0):
    """ Enables acceptance and processing of passive host checks for the specified host. """
    return send_command("ENABLE_PASSIVE_HOST_CHECKS",command_file,timestamp, host_name)

def disable_passive_host_checks(host_name,command_file=None,timestamp=0):
    """ Disables acceptance and processing of passive host checks for the specified host. """
    return send_command("DISABLE_PASSIVE_HOST_CHECKS",command_file,timestamp, host_name)

def enable_flap_detection(command_file=None,timestamp=0):
    """ Enables host and service flap detection on a program-wide basis. """
    return send_command("ENABLE_FLAP_DETECTION",command_file,timestamp, )

def disable_flap_detection(command_file=None,timestamp=0):
    """ Disables host and service flap detection on a program-wide basis. """
    return send_command("DISABLE_FLAP_DETECTION",command_file,timestamp, )

def enable_host_flap_detection(host_name,command_file=None,timestamp=0):
    """ Enables flap detection for the specified host.  In order for the flap detection algorithms to be run for the host, flap detection must be enabled on a program-wide basis as well. """
    return send_command("ENABLE_HOST_FLAP_DETECTION",command_file,timestamp, host_name)

def enable_svc_flap_detection(host_name,service_description,command_file=None,timestamp=0):
    """ Enables flap detection for the specified service.  In order for the flap detection algorithms to be run for the service, flap detection must be enabled on a program-wide basis as well. """
    return send_command("ENABLE_SVC_FLAP_DETECTION",command_file,timestamp, host_name,service_description)

def disable_host_flap_detection(host_name,command_file=None,timestamp=0):
    """ Disables flap detection for the specified host. """
    return send_command("DISABLE_HOST_FLAP_DETECTION",command_file,timestamp, host_name)

def disable_service_flap_detection(host_name,service_description,command_file=None,timestamp=0):
    """ Disables flap detection for the specified service. """
    return send_command("DISABLE_SERVICE_FLAP_DETECTION",command_file,timestamp, host_name,service_description)

def enable_hostgroup_svc_notifications(hostgroup_name,command_file=None,timestamp=0):
    """ Enables notifications for all services that are associated with hosts in a particular hostgroup.  This does not enable notifications for the hosts in the hostgroup - see the ENABLE_HOSTGROUP_HOST_NOTIFICATIONS command for that.  In order for notifications to be sent out for these services, notifications must be enabled on a program-wide basis as well. """
    return send_command("ENABLE_HOSTGROUP_SVC_NOTIFICATIONS",command_file,timestamp, hostgroup_name)

def enable_notifications(command_file=None,timestamp=0):
    """ Enables host and service notifications on a program-wide basis. """
    return send_command("ENABLE_NOTIFICATIONS",command_file,timestamp, )

def disable_hostgroup_svc_notifications(hostgroup_name,command_file=None,timestamp=0):
    """ Disables notifications for all services associated with hosts in a particular hostgroup.  This does not disable notifications for the hosts in the hostgroup - see the DISABLE_HOSTGROUP_HOST_NOTIFICATIONS command for that. """
    return send_command("DISABLE_HOSTGROUP_SVC_NOTIFICATIONS",command_file,timestamp, hostgroup_name)

def enable_hostgroup_host_notifications(hostgroup_name,command_file=None,timestamp=0):
    """ Enables notifications for all hosts in a particular hostgroup.  This does not enable notifications for the services associated with the hosts in the hostgroup - see the ENABLE_HOSTGROUP_SVC_NOTIFICATIONS command for that.  In order for notifications to be sent out for these hosts, notifications must be enabled on a program-wide basis as well. """
    return send_command("ENABLE_HOSTGROUP_HOST_NOTIFICATIONS",command_file,timestamp, hostgroup_name)

def disable_hostgroup_host_notifications(hostgroup_name,command_file=None,timestamp=0):
    """ Disables notifications for all hosts in a particular hostgroup.  This does not disable notifications for the services associated with the hosts in the hostgroup - see the DISABLE_HOSTGROUP_SVC_NOTIFICATIONS command for that. """
    return send_command("DISABLE_HOSTGROUP_HOST_NOTIFICATIONS",command_file,timestamp, hostgroup_name)

def enable_hostgroup_svc_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Enables active checks for all services associated with hosts in a particular hostgroup. """
    return send_command("ENABLE_HOSTGROUP_SVC_CHECKS",command_file,timestamp, hostgroup_name)

def disable_hostgroup_svc_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Disables active checks for all services associated with hosts in a particular hostgroup. """
    return send_command("DISABLE_HOSTGROUP_SVC_CHECKS",command_file,timestamp, hostgroup_name)

def enable_hostgroup_host_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Enables active checks for all hosts in a particular hostgroup. """
    return send_command("ENABLE_HOSTGROUP_HOST_CHECKS",command_file,timestamp, hostgroup_name)

def disable_hostgroup_host_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Disables active checks for all hosts in a particular hostgroup. """
    return send_command("DISABLE_HOSTGROUP_HOST_CHECKS",command_file,timestamp, hostgroup_name)

def enable_hostgroup_passive_host_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Enables passive checks for all hosts in a particular hostgroup. """
    return send_command("ENABLE_HOSTGROUP_PASSIVE_HOST_CHECKS",command_file,timestamp, hostgroup_name)

def disable_hostgroup_passive_host_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Disables passive checks for all hosts in a particular hostgroup. """
    return send_command("DISABLE_HOSTGROUP_PASSIVE_HOST_CHECKS",command_file,timestamp, hostgroup_name)

def enable_hostgroup_passive_svc_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Enables passive checks for all services associated with hosts in a particular hostgroup. """
    return send_command("ENABLE_HOSTGROUP_PASSIVE_SVC_CHECKS",command_file,timestamp, hostgroup_name)

def restart_program(command_file=None,timestamp=0):
    """ Restarts the Nagios process. """
    return send_command("RESTART_PROGRAM",command_file,timestamp, )

def disable_hostgroup_passive_svc_checks(hostgroup_name,command_file=None,timestamp=0):
    """ Disables passive checks for all services associated with hosts in a particular hostgroup. """
    return send_command("DISABLE_HOSTGROUP_PASSIVE_SVC_CHECKS",command_file,timestamp, hostgroup_name)

def enable_servicegroup_svc_notifications(servicegroup_name,command_file=None,timestamp=0):
    """ Enables notifications for all services that are members of a particular servicegroup.  In order for notifications to be sent out for these services, notifications must also be enabled on a program-wide basis. """
    return send_command("ENABLE_SERVICEGROUP_SVC_NOTIFICATIONS",command_file,timestamp, servicegroup_name)

def disable_servicegroup_svc_notifications(servicegroup_name,command_file=None,timestamp=0):
    """ Disables notifications for all services that are members of a particular servicegroup. """
    return send_command("DISABLE_SERVICEGROUP_SVC_NOTIFICATIONS",command_file,timestamp, servicegroup_name)

def enable_servicegroup_host_notifications(servicegroup_name,command_file=None,timestamp=0):
    """ Enables notifications for all hosts that have services that are members of a particular servicegroup.  In order for notifications to be sent out for these hosts, notifications must also be enabled on a program-wide basis. """
    return send_command("ENABLE_SERVICEGROUP_HOST_NOTIFICATIONS",command_file,timestamp, servicegroup_name)

def disable_servicegroup_host_notifications(servicegroup_name,command_file=None,timestamp=0):
    """ Disables notifications for all hosts that have services that are members of a particular servicegroup. """
    return send_command("DISABLE_SERVICEGROUP_HOST_NOTIFICATIONS",command_file,timestamp, servicegroup_name)

def enable_servicegroup_svc_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Enables active checks for all services in a particular servicegroup. """
    return send_command("ENABLE_SERVICEGROUP_SVC_CHECKS",command_file,timestamp, servicegroup_name)

def disable_servicegroup_svc_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Disables active checks for all services in a particular servicegroup. """
    return send_command("DISABLE_SERVICEGROUP_SVC_CHECKS",command_file,timestamp, servicegroup_name)

def enable_servicegroup_host_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Enables active checks for all hosts that have services that are members of a particular hostgroup. """
    return send_command("ENABLE_SERVICEGROUP_HOST_CHECKS",command_file,timestamp, servicegroup_name)

def disable_servicegroup_host_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Disables active checks for all hosts that have services that are members of a particular hostgroup. """
    return send_command("DISABLE_SERVICEGROUP_HOST_CHECKS",command_file,timestamp, servicegroup_name)

def enable_servicegroup_passive_svc_checks(servicegroup_name,command_file=None,timestamp=0):
    """ Enables the acceptance and processing of passive checks for all services in a particular servicegroup. """
    return send_command("ENABLE_SERVICEGROUP_PASSIVE_SVC_CHECKS",command_file,timestamp, servicegroup_name)



1# -*- coding: utf-8 -*-
#
# pynag - Python Nagios plug-in and configuration environment
# Copyright (C) 2012 Pall Sigurdsson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
These are helper functions and implementation of proposed new threshold format for nagios plugins
according to: http://nagiosplugins.org/rfc/new_threshold_syntax

In short, plugins should implement a --threshold option which takes argument in form of:
  # metric={metric},ok={range},warn={range},crit={range},unit={unit}prefix={SI prefix}

Example:
  --treshold metric=load1,ok=0..5,warning=5..10,critical=10..inf

"""
import pynag.Plugins
from pynag.Utils import PynagError


def check_threshold(value, ok=None, warning=None, critical=None):
    """ Checks value against warning/critical and returns Nagios exit code.

    Format of range_threshold is according to:
    http://nagiosplugins.org/rfc/new_threshold_syntax

    Returns (in order of appearance):
        UNKNOWN int(3)  -- On errors or bad input
        OK int(0)       -- If value is within ok threshold
        CRITICAL int(2) -- if value is within critical threshold
        WARNING int(1)  -- If value is within warning threshold
        OK int(0)       -- If value is outside both warning and critical
        OK int(0)       -- If no thresholds are provided
    Arguments:
        value    -- value to check
        ok       -- ok range
        warning  -- warning range
        critical -- critical range

    # Example Usage:
    >>> check_threshold(88, warning="90..95", critical="95..100")
    0
    >>> check_threshold(92, warning="90..95", critical="95..100")
    1
    >>> check_threshold(96, warning="90..95", critical="95..100")
    2
    """
    if ok and check_range(value, ok):
        return OK
    elif critical and check_range(value, critical):
        return CRITICAL
    elif warning and check_range(value, warning):
        return WARNING
    else:
        return OK


def check_range(value, range):
    """ Returns True if value is within range, else False

    Arguments:
      value -- Numerical value to check, can be any number
      range -- string in the format of "start..end"
    Examples:
    >>> check_range(5, "0..10")
    True
    >>> check_range(11, "0..10")
    False
    """

    if not type(range) == type('') or range == '':
        raise PynagError('range must be a string')
        # value must be numeric, so we try to convert it to float
    value = float(value)

    # If range starts with ^ we the conditions are inverted
    if range[0] == '^':
        return not check_range(value, range[1:])

    # Start and end must be defined
    tmp = range.split('..')
    if len(tmp) != 2:
        raise PynagError('Invalid Format for threshold range: "%s"' % range)
    start,end = tmp

    if not start in ('inf','-inf'):
        start = float(start)
        if start > value:
            return False
    if not end == 'inf':
        end = float(end)
        if end < value:
            return False
    return True


def parse_threshold(threshold):
    """ takes a threshold string as an input and returns a hash map of options and values

    Examples:
        >>> parse_threshold('metric=disk_usage,ok=0..90,warning=90..95,critical=95.100")
        {'metric':'disk_usage', 'thresholds':[(0,'0..90'), (1,'90..95'),(2,'95..100')]}
    """
    tmp = threshold.lower().split(',')
    parsed_thresholds = []
    metric_name=None
    results = {}
    results['thresholds'] = parsed_thresholds
    for i in tmp:
        if i.find('=') < 1:
            raise PynagError("Invalid input: '%s' is not of the format key=value" % i)
        key,value = i.split('=',1)
        if key in pynag.Plugins.state.keys():
            parsed_thresholds.append( (pynag.Plugins.state[key], value) )
        else:
            results[key] = value
    return results



#!/usr/bin/python
#
# This script will autogenerate python functions to communicate with the python command file.
# input to the program is the documentation from nagios

import BeautifulSoup
import sys

for filename in sys.argv[1:]:
    html = open(filename).read()

    soup = BeautifulSoup.BeautifulSoup(html,  convertEntities='html')

    # First get the command format we need:
    tmp = soup.find('td', {'class':'MediumBold'})
    command_format = tmp.findNext().getText()

    # command_format should look something like this:
    # u'ENABLE_SVC_EVENT_HANDLER;&lt;host_name&gt;;&lt;service_description&gt;'
    command_format = command_format.replace('<', '',).replace('>','')
    command_format = command_format.split(';')
    func = command_format[0]
    # Lets convert function name to lowercase to be polite
    args = command_format[1:]


    # Now we have the command format, lets find the description text
    description = tmp.findParent().findNextSibling().findNextSibling().findNextSibling().findNextSibling().getText()


    strFunction = """
def %s(%s,command_file=None,timestamp=0):
    \"\"\" %s \"\"\"
    return send_command("%s",command_file,timestamp, %s)"""
    strFunction = strFunction % (func.lower(), ','.join(args), description, func, ','.join(args))
    strFunction = strFunction.replace('(,','(')
    print strFunction



# -*- coding: utf-8 -*-
#
# pynag - Python Nagios plug-in and configuration environment
# Copyright (C) 2010 Drew Stinnet
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Python Nagios extensions
"""

import sys
import os
import re
import traceback
from platform import node
from optparse import OptionParser,OptionGroup
from pynag.Utils import PerfData,PerfDataMetric
import new_threshold_syntax

# Map the return codes
OK = 0
WARNING = 1
CRITICAL = 2
UNKNOWN = 3

ok,warning,critical,unknown = 0,1,2,3

state = {}
state['ok'] = 0
state['warning'] = 1
state['warn'] = 1
state['w'] = 1
state['critical'] = 2
state['crit'] = 2
state['c'] = 2
state['unknown'] = 3
state['u'] = 3


state_text = {}
state_text[ok] = 'OK'
state_text[warning] = 'Warning'
state_text[critical] = "Critical"
state_text[unknown] = "Unknown"

class simple:
    """
    Nagios plugin helper library based on Nagios::Plugin

    Sample usage

    from pynag.Plugins import WARNING, CRITICAL, OK, UNKNOWN, simple as Plugin

    # Create plugin object
    np = Plugin()
    # Add arguments
    np.add_arg("d", "disk")
    # Do activate plugin
    np.activate()
    ... check stuff, np['disk'] to address variable assigned above...
    # Add a status message and severity
    np.add_message( WARNING, "Disk nearing capacity" )
    # Get parsed code and messages
    (code, message) = np.check_messages()
    # Return information and exit
    nagios_exit(code, message)
    """

    def __init__(self, shortname = None, version = None, blurb = None, extra = None, url = None, license = None, plugin = None, timeout = 15, must_threshold = True):

        ## this is the custom parser
        self.extra_list_optional = []
        self.extra_list_required = []

        ## Set the option parser stuff here
        self.parser = OptionParser()

        ## Variables we'll get later
        self.opts = None
        self.must_threshold = must_threshold
        self.data = {}
        self.data['perfdata'] = []
        self.data['messages'] = { OK:[], WARNING:[], CRITICAL:[], UNKNOWN:[] }
        self.data['threshhold'] = None

        ## Error mappings, for easy access
        self.errors = { "OK":0, "WARNING":1, "CRITICAL":2, "UNKNOWN":3, }
        self.status_text = { 0:"OK", 1:"WARNING", 2:"CRITICAL", 3:"UNKNOWN", }

        ## Shortname creation
        if not shortname:
            self.data['shortname'] = os.path.basename("%s" % sys.argv[0])
        else:
            self.data['shortname'] = shortname

    def add_arg(self, spec_abbr, spec, help_text, required=1, action="store"):
        """
        Add an argument to be handled by the option parser.  By default, the arg is not required.
        
        required = optional parameter
        action = [store, append, store_true]
        """
        self.parser.add_option("-%s" % spec_abbr, "--%s" % spec, dest="%s" % spec, help=help_text, metavar="%s" % spec.upper(), action=action)
        if required:
            self.extra_list_required.append(spec)
        else:
            self.extra_list_optional.append(spec)

    def activate(self):
        """
        Parse out all command line options and get ready to process the plugin.  This should be run after argument preps
        """
        timeout = None
        verbose = 0

        self.parser.add_option("-v", "--verbose", dest="verbose", help="Verbosity Level", metavar="VERBOSE", default=0)
        self.parser.add_option("-H", "--host", dest="host", help="Target Host", metavar="HOST")
        self.parser.add_option("-t", "--timeout", dest="timeout", help="Connection Timeout", metavar="TIMEOUT")

        if self.must_threshold == True:
            self.parser.add_option("-c", "--critical", dest="critical", help="Critical Threshhold", metavar="CRITICAL")
            self.parser.add_option("-w", "--warning", dest="warning", help="Warn Threshhold", metavar="WARNING")

        (options, args) = self.parser.parse_args()

        ## Set verbosity level
        if int(options.verbose) in (0, 1, 2, 3):
            self.data['verbosity'] = options.verbose
        else:
            self.data['verbosity'] = 0

        ## Ensure the hostname is set
        if options.host:
            self.data['host'] = options.host

        ## Set timeout
        if options.timeout:
            self.data['timeout'] = options.timeout
        else:
            self.data['timeout'] = timeout

        if self.must_threshold == True and not options.critical and not options.warning:
            self.parser.error("You must provide a WARNING and/or CRITICAL value")

        ## Set Critical; if the option is available in the plugin
        if hasattr(options, 'critical'):
            self.data['critical'] = options.critical
        else:
            self.data['critical'] = None

        ## Set Warn; if the option is available in the plugin
        if hasattr(options, 'warning'):
            self.data['warning'] = options.warning
        else:
            self.data['warning'] = None

        ## Ensurethat the extra items are provided
        for extra_item in self.extra_list_required:
            if not options.__dict__[extra_item]:
                self.parser.error("option '%s' is required" % extra_item)


        ## Put the remaining values into the data dictionary
        for key,value in options.__dict__.items():
            if key in (self.extra_list_required + self.extra_list_optional):
                self.data[key] = value

    def add_perfdata(self, label , value , uom = None, warn = None, crit = None, minimum = None, maximum = None):
        """
        Append perfdata string to the end of the message
        """

        # Append perfdata (assume multiple)
        self.data['perfdata'].append({ 'label' : label, 'value' : value, 'uom' : uom, 
            'warn' : warn, 'crit' : crit, 'min' : minimum, 'max' : maximum})

    def check_range(self, value):
        """
        Check if a value is within a given range.  This should replace change_threshold eventually. Exits with appropriate exit code given the range.

        Taken from:  http://nagiosplug.sourceforge.net/developer-guidelines.html
        Range definition
    
        Generate an alert if x...
        10        < 0 or > 10, (outside the range of {0 .. 10})
        10:        < 10, (outside {10 .. #})
        ~:10    > 10, (outside the range of {-# .. 10})
        10:20    < 10 or > 20, (outside the range of {10 .. 20})
        @10:20    # 10 and # 20, (inside the range of {10 .. 20})
        """
        critical = self.data['critical']
        warning = self.data['warning']
        self.hr_range = ""

        if critical and not self._range_checker(value, critical):
            self.add_message(CRITICAL,"%s is outside critical range: %s" % (value, critical))
        elif warning and not self._range_checker(value, warning):
            self.add_message(WARNING,"%s is outside warning range: %s" % (value, warning))
        else:
            self.add_message(OK,"%s is inside warning=%s and critical=%s" % (value, warning, critical))

        # Get all messages appended and exit code
        (code, message) = self.check_messages()

        # Exit with appropriate exit status and message
        self.nagios_exit(code, message)

    def _range_checker(self, value, range_threshold):
        """ deprecated. Use pynag.Plugins.check_range() """
        return check_range(value=value, range_threshold=range_threshold)

    def send_nsca(self, code, message, ncsahost, hostname=node(), service=None):
        """
        Send data via send_nsca for passive service checks
        """
    
        # Execute send_nsca
        from popen2 import Popen3
        command = "send_nsca -H %s" % ncsahost
        p = Popen3(command,  capturestderr=True)

        # Service check
        if service:
            print >>p.tochild, "%s    %s    %s    %s %s" % (hostname, service, code, message, self.perfdata_string())
        # Host check, omit service_description
        else:
            print >>p.tochild, "%s    %s    %s %s" % (hostname, code, message, self.perfdata_string())

        # Send eof
        # TODO, support multiple statuses ?
        p.tochild.close()

        # Save output incase we have an error
        nsca_output = ''
        for line in p.fromchild.readlines():
            nsca_output += line

        # Wait for send_nsca to exit
        returncode = p.wait()
        returncode = os.WEXITSTATUS( returncode) 

        # Problem with running nsca
        if returncode != 0:
            if returncode == 127:
                raise Exception("Could not find send_nsca in path")
            else:
                raise Exception("returncode: %i\n%s" % (returncode, nsca_output))

        return 0

    def nagios_exit(self, code_text, message):
        """
        Exit with exit_code, message, and optionally perfdata
        """

        # Change text based codes to int
        code = self.code_string2int(code_text)

        ## This should be one line (or more in nagios 3)
        print "%s: %s %s" % (self.status_text[code], message, self.perfdata_string())
        sys.exit(code)

    def perfdata_string(self):

        ## Append perfdata to the message, if perfdata exists
        if self.data['perfdata']:
            append = '|'
        else:
            append = ''

        for pd in self.data['perfdata']:
            append += " '%s'=%s%s;%s;%s;%s;%s" % (
                pd['label'],
                pd['value'],
                pd['uom'] or '',
                pd['warn'] or '',
                pd['crit'] or '',
                pd['min'] or '',
                pd['max'] or '')

        return append

    def add_message( self, code, message ):
        """
        Add a message with code to the object. May be called
        multiple times.  The messages added are checked by check_messages,
        following.

        Only CRITICAL, WARNING, OK and UNKNOWN are accepted as valid codes.
        """
        # Change text based codes to int
        code = self.code_string2int(code)

        self.data['messages'][code].append( message )

    def check_messages( self, joinstr = " ", joinallstr = None ):
        """
        Check the current set of messages and return an appropriate nagios
        return code and/or a result message. In scalar context, returns
        only a return code; in list context returns both a return code and
        an output message, suitable for passing directly to nagios_exit()

        joinstr = string
            A string used to join the relevant array to generate the
            message string returned in list context i.e. if the 'critical'
            array is non-empty, check_messages would return:
                joinstr.join(critical)

        joinallstr = string
            By default, only one set of messages are joined and returned in
            the result message i.e. if the result is CRITICAL, only the
            'critical' messages are included in the result; if WARNING,
            only the 'warning' messages are included; if OK, the 'ok'
            messages are included (if supplied) i.e. the default is to
            return an 'errors-only' type message.

            If joinallstr is supplied, however, it will be used as a string
            to join the resultant critical, warning, and ok messages
            together i.e.  all messages are joined and returned.
        """
        # Check for messages in unknown, critical, warning, ok to determine
        # code
        keys = self.data['messages'].keys()
        keys.sort(reverse=True)
        code = UNKNOWN
        for code in keys:
            if len(self.data['messages'][code]):
                break

        # Create the relevant message for the most severe code
        if joinallstr is None:
            message = joinstr.join(self.data['messages'][code])
        # Join all strings whether OK, WARN...
        else:
            message = ""
            for c in keys:
                if len(self.data['messages'][c]):
                    message += joinallstr.join(self.data['messages'][c]) + joinallstr

        return code, message.rstrip(joinallstr)

    def code_string2int( self, code_text ):
        """
        Changes CRITICAL, WARNING, OK and UNKNOWN code_text to integer
        representation for use within add_message() and nagios_exit()
        """

        # If code_text is a string, convert to the int
        if str(type(code_text)) == "<type 'str'>":
            code = self.errors[code_text]
        else:
            code = code_text

        return code

    def __setitem__(self, key, item):
        self.data[key] = item

    def __getitem__(self, key):
        if key in self.data:
            return self.data[key]
        else:
            return None

def check_threshold(value, warning=None, critical=None):
    """ Checks value against warning/critical and returns Nagios exit code.

    Format of range_threshold is according to:
    http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT

    Returns (in order of appearance):
        UNKNOWN int(3)  -- On errors or bad input
        CRITICAL int(2) -- if value is within critical threshold
        WARNING int(1)  -- If value is within warning threshold
        OK int(0)       -- If value is outside both tresholds
    Arguments:
        value -- value to check
        warning -- warning range
        critical -- critical range

    # Example Usage:
    >>> check_threshold(88, warning="0:90", critical="0:95")
    0
    >>> check_threshold(92, warning=":90", critical=":95")
    1
    >>> check_threshold(96, warning=":90", critical=":95")
    2
    """
    if critical and not check_range(value, critical):
        return CRITICAL
    elif warning and not check_range(value, warning):
        return WARNING
    else:
        return OK


def check_range(value, range_threshold=None):
    """ Returns True if value is within range_threshold.

    Format of range_threshold is according to:
    http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT

    Arguments:
        value -- Numerical value to check (i.e. 70 )
        range -- Range to compare against (i.e. 0:90 )
    Returns:
        True  -- If value is inside the range
        False -- If value is outside the range (alert if this happens)

    Summary from plugin developer guidelines:
    ---------------------------------------------------------
    x       Generate an alert if x...
    ---------------------------------------------------------
    10  	< 0 or > 10, (outside the range of {0 .. 10})
    10:     < 10, (outside {10 .. ∞})
    ~:10    > 10, (outside the range of {-∞ .. 10})
    10:20   < 10 or > 20, (outside the range of {10 .. 20})
    @10:20  ≥ 10 and ≤ 20, (inside the range of {10 .. 20})
    10      < 0 or > 10, (outside the range of {0 .. 10})
    ---------------------------------------------------------


    # Example runs for doctest, False should mean alert
    >>> check_range(78, "90") # Example disk is 78% full, threshold is 90
    True
    >>> check_range(5, 10) # Everything between 0 and 10 is True
    True
    >>> check_range(0, 10) # Everything between 0 and 10 is True
    True
    >>> check_range(10, 10) # Everything between 0 and 10 is True
    True
    >>> check_range(11, 10) # Everything between 0 and 10 is True
    False
    >>> check_range(-1, 10) # Everything between 0 and 10 is True
    False
    >>> check_range(-1, "~:10") # Everything Below 10
    True
    >>> check_range(11, "10:") # Everything above 10 is True
    True
    >>> check_range(1, "10:") # Everything above 10 is True
    False
    >>> check_range(0, "5:10") # Everything between 5 and 10 is True
    False
    >>> check_range(0, "@5:10") # Everything outside 5:10 is True
    True
    """

    # if no range_threshold is provided, assume everything is ok
    if not range_threshold:
        range_threshold='~:'
    range_threshold = str(range_threshold)
    # If range starts with @, then we do the opposite
    if range_threshold[0] == '@':
        return not check_range(value, range_threshold[1:])

    value = float(value)
    if range_threshold.find(':') > -1:
        (start,end) = (range_threshold.split(':', 1))
    # we get here if ":" was not provided in range_threshold
    else:
        start = ''
        end = range_threshold
    # assume infinity if start is not provided
    if start == '~':
        start = None
    # assume start=0 if start is not provided
    if start == '':
        start = 0
    # assume infinity if end is not provided
    if end == '':
        end = None
    # start is defined and value is lower than start
    if start is not None and float(value) < float(start):
        return False
    if end is not None and float(value) > float(end):
        return False
    return True


class PluginHelper:
    """ PluginHelper takes away some of the tedious work of writing Nagios plugins. Primary features include:

    * Keep a collection of your plugin messages (queue for both summary and longoutput)
    * Keep record of exit status
    * Keep a collection of your metrics (for both perfdata and thresholds)
    * Automatic Command-line arguments
    * Make sure output of your plugin is within Plugin Developer Guidelines

    Usage:
    p = PluginHelper()
    p.status(warning)
    p.add_summary('Example Plugin with warning status')
    p.add_metric('cpu load', '90')
    p.exit()
    """
    _nagios_status = -1     # exit status of the plugin
    _long_output = []       # Long output of the plugin
    _summary = []           # Summary of the plugin
    _perfdata = PerfData()  # Performance and Threshold Metrics are stored here
    show_longoutput = True  # If True, print longoutput
    show_perfdata = True    # If True, print perfdata
    show_summary = True     # If True, print Summary
    show_status_in_summary = False
    verbose = False         # Extra verbosity
    show_debug = False      # Extra debugging

    thresholds = [] # List of strings in the nagios threshold format
    options = None          # OptionParser() options
    arguments = None        # OptionParser() arguments

    def __init__(self):
        self.parser = OptionParser()
        general = OptionGroup(self.parser, "Generic Options")
        self.parser.add_option('--threshold','--th',default=[], help="Thresholds in standard nagios threshold format", metavar='', dest="thresholds",action="append")

        display_group = OptionGroup(self.parser, "Display Options")
        display_group.add_option("-v", "--verbose", dest="verbose", help="Print more verbose info", metavar="v", action="store_true", default=False)
        general.add_option("-d", "--debug", dest="show_debug", help="Print debug info", metavar="d", action="store_true", default=False)
        display_group.add_option("--no-perfdata", dest="show_perfdata", help="Dont show any performance data", action="store_false", default=True)
        display_group.add_option("--no-longoutput", dest="show_longoutput", help="Hide longoutput from the plugin output (i.e. only display first line of the output)", action="store_false", default=True)
        display_group.add_option("--no-summary", dest="show_summary", help="Hide summary from plugin output", action="store_false", default=True)
        #display_group.add_option("--show-status-in-summary", dest="show_status_in_summary", help="Prefix the summary of the plugin with OK- or WARN- ", action="store_true", default=False)
        display_group.add_option("--get-metrics", dest="get_metrics", help="Print all available metrics and exit (can be combined with --verbose)", action="store_true", default=False)

        self.parser.add_option_group(display_group)

    def parse_arguments(self, argument_list=None):
        """ Parsers commandline arguments, prints error if there is a syntax error.

        Creates:
            self.options   -- As created by OptionParser.parse()
            self.arguments -- As created by OptionParser.parse()
        Arguments:
            argument_list -- By default use sys.argv[1:], override only if you know what you are doing.
        Returns:
            None
        """
        self.options, self.arguments = self.parser.parse_args(args=argument_list)
        # TODO: Handle it if developer decides to remove some options before calling parse_arguments()
        self.thresholds = self.options.thresholds
        self.show_longoutput = self.options.show_longoutput
        self.show_perfdata = self.options.show_perfdata
        self.show_debug = self.options.show_debug
        self.verbose = self.options.verbose
        #self.show_status_in_summary = self.options.show_status_in_summary

    def add_long_output(self, message):
        """ Appends message to the end of Plugin long_output. Message does not need a \n suffix

        Examples:
          >>> p = PluginHelper()
          >>> p.add_long_output('Status of sensor 1')
          >>> p.add_long_output('* Temperature: OK')
          >>> p.add_long_output('* Humidity: OK')
          >>> p.get_long_output()
          'Status of sensor 1\\n* Temperature: OK\\n* Humidity: OK'
        """
        self._long_output.append(message)

    def get_long_output(self):
        """ Returns all long_output that has been added via add_long_output """
        return '\n'.join(self._long_output)

    def add_summary(self, message):
        """ Adds message to Plugin Summary """
        self._summary.append(message.strip())

    def get_summary(self):
        return '. '.join(self._summary)

    def get_status(self):
        """ Returns the worst nagios status (integer 0,1,2,3) that has been put with add_status()

        If status has never been added, returns 3 for UNKNOWN
        """

        # If no status has been set, return unknown
        if self._nagios_status == -1:
            return UNKNOWN
        else:
            return self._nagios_status

    def status(self, new_status=None):
        """ Same as get_status() if new_status=None, otherwise call add_status(new_status) """
        if new_status is None:
            return self.get_status()
        return self.add_status(new_status)

    def add_status(self, new_status=None):
        """ Update exit status of the nagios plugin. This function will keep history of the worst status added

        Examples:
        >>> p = PluginHelper()
        >>> p.add_status(0) # ok
        >>> p.add_status(2) # critical
        >>> p.add_status(1) # warning
        >>> p.get_status()  #
        2
        """

        # If new status was entered as a human readable string (ok,warn,etc) lets convert it to int:
        if type(new_status) == type('') and new_status.lower() in state:
            new_status = state[new_status]

        self._nagios_status = max(self._nagios_status, new_status)

    def add_metric(self, label="",value="",warn="",crit="",min="",max="",uom="", perfdatastring=None):
        """ Add numerical metric (will be outputted as nagios performanca data)

        Examples:
          >>> p = PluginHelper()
          >>> p.add_metric(label="load1", value="7")
          >>> p.add_metric(label="load5", value="5")
          >>> p.add_metric(label="load15",value="2")
          >>> p.get_perfdata()
          "'load1'=7;;;; 'load5'=5;;;; 'load15'=2;;;;"
        """
        if not perfdatastring is None:
            self._perfdata.add_perfdatametric(perfdatastring=perfdatastring)
        else:
            self._perfdata.add_perfdatametric(label=label,value=value,warn=warn,crit=crit,min=min,max=max,uom=uom)

    def get_metric(self, label):
        """ Return one specific metric (PerfdataMetric object) with the specified label. Returns None if not found. """
        for i in self._perfdata.metrics:
            if i.label == label:
                return i
        return None

    def get_perfdata(self):
        """ Get perfdatastring for all valid perfdatametrics collected via add_perfdata """
        return str(self._perfdata   )

    def get_plugin_output(self, exit_code=None,summary=None, long_output=None, perfdata=None):
        """ Get all plugin output as it would be printed to screen with self.exit() """
        if summary is None:
            summary = self.get_summary()
        if long_output is None:
            long_output = self.get_long_output()
        if perfdata is None:
            perfdata = self.get_perfdata()
        if exit_code is None:
            exit_code = self.get_status()

        return_buffer = ""
        if self.show_status_in_summary == True:
            return_buffer += "%s - " % state_text[exit_code]
        if self.show_summary == True:
            return_buffer += summary
        if self.show_perfdata == True and len(perfdata) > 0:
            return_buffer += " | %s\n" % perfdata

        if not return_buffer.endswith('\n'):
            return_buffer += '\n'
        if self.show_longoutput == True and len(long_output) > 0:
            return_buffer += long_output

        return_buffer = return_buffer.strip()
        return return_buffer

    def exit(self, exit_code=None,summary=None, long_output=None, perfdata=None):
        """ Print all collected output to screen and exit nagios style, no arguments are needed
            except if you want to override default behavior.

        Arguments:
            summary     -- Is this text as the plugin summary instead of self.get_summary()
            long_output -- Use this text as long_output instead of self.get_long_output()
            perfdata    -- Use this text instead of self.get_perfdata()
            exit_code   -- Use this exit code instead of self.status()
        """
        if exit_code is None:
            exit_code = self.get_status()
        if self.options.get_metrics == True:
            summary = "Available metrics for this plugin:"
            metrics = []

            for i in self._perfdata.metrics:
                if self.options.verbose == True:
                    metrics.append( str(i) )
                else:
                    metrics.append( i.label )
            long_output = '\n'.join(metrics)


        plugin_output = self.get_plugin_output(exit_code=exit_code,summary=summary,long_output=long_output,perfdata=perfdata)

        print plugin_output
        sys.exit(exit_code)

    def check_metric(self, metric_name, thresholds):
        """ Check one specific metric against a list of thresholds. Updates self.status() and writes to summary or longout as appropriate.

        Arguments:
          metric_name -- A string representing the name of the metric (the label part of the performance data)
          thresholds  -- a list in the form of [ (level,range) ] where range is a string in the format of "start..end"

        Examples:
        >>> thresholds = [(warning,'2..5'),(critical,'5..inf')]
        >>> p = PluginHelper()
        >>> p.check_metric('load15',thresholds)

        Returns:
          None
        """
        metric = self.get_metric(label=metric_name)

        # If threshold was specified but metric not found in our data, set status unknown
        if metric is None:
            self.status(unknown)
            self.add_summary("Metric %s not found" % (metric_name))
            return

        metric_status = OK # by default assume status is ok
        for level,threshold_range in thresholds:
            if metric.warn == '' and level == warning:
                metric.warn = threshold_range
            elif metric.crit == '' and level == critical:
                metric.crit = threshold_range
            if new_threshold_syntax.check_range(metric.value, threshold_range):
                metric_status = max(metric_status, level)
                self.debug('%s is within %s range "%s"' % (metric_name, state_text[level], threshold_range))
            else:
                self.debug('%s is outside %s range "%s"' % (metric_name, state_text[level],threshold_range))

        # OK's go to long output, errors go directly to summary
        self.add_status(metric_status)
        message = '%s on %s' % (state_text[metric_status], metric_name)

        # Errors are added to the summary:
        if metric_status > 0:
            self.add_summary(message)

        if self.verbose == True:
            self.add_long_output(message)


    def check_all_metrics(self):
        """ Checks all metrics (add_metric() against any thresholds set in self.options.thresholds or with --threshold from commandline)"""
        checked_metrics = []
        for threshold in self.thresholds:
            parsed_threshold = new_threshold_syntax.parse_threshold(threshold)
            metric_name = parsed_threshold['metric']
            thresholds = parsed_threshold['thresholds']
            self.check_metric(metric_name, thresholds)
            checked_metrics.append( metric_name )

        # Lets look at metrics that were not specified on the command-line but might have a default
        # threshold specified with their metric data
        for i in self._perfdata.metrics:
            if i.label in checked_metrics:
                continue
            thresholds = []

            if i.warn != '':
                thresholds.append( (warning, i.warn))
            if i.crit != '':
                thresholds.append( (critical, i.crit))
            self.check_metric(i.label, thresholds)

    def run_function(self, function):
        """ Executes "function" and exits nagios style if there are any exceptions."""
        try:
            function()
        except Exception, e:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            exit_code = unknown
            #traceback.print_exc(file=sys.stdout)
            summary = "Unhandled '%s' exception while running plugin (traceback below)" % exc_type
            long_output =traceback.format_exc()
            self.exit(exit_code=exit_code, summary=summary, long_output=long_output,perfdata='')

    def debug(self, message):
        if self.show_debug == True:
            self.add_long_output("debug: %s" % message)


    def __str__(self):
        return self.get_plugin_output()

    def __repr__(self):
        return self.get_plugin_output(long_output='',perfdata='')
